import { signal, computed } from '@angular/core';
import { produce } from 'immer';
import { deepEqual, shallowEqual } from 'fast-equals';

/** Create a `Signal` with immutable state that can be set or updated directly.*/
function immutableSignal(...args) {
    const [value, ...rest] = args;
    const s = signal(produce(value, (v) => v), ...rest);
    return new Proxy(s, {
        get(target, property, receiver) {
            switch (property) {
                case 'mutate':
                    return (mutatorFn) => {
                        return target.update((val) => produce(val, mutatorFn));
                    };
                case 'set':
                    return (val) => {
                        return target.set(produce(val, (v) => v));
                    };
                case 'update':
                    return (updateFn) => {
                        return target.update((val) => produce(val, updateFn));
                    };
                default:
                    return Reflect.get(target, property, receiver);
            }
        },
    });
}

/** Create a computed Signal and provides a predefined equality function.
 * @param equalityDefinition
 * A string that determines the behaviour of the equality function.
 * * `default` - Checks for equality using the `===` operator
 * * `shallow` - Performs default check. if false, compares values based on their first depth of values.
 * * `deep` - Performs default check. if false, compares values based on their full depth of values.
 */
function derived(computation, equalityDefinition = 'default') {
    let equal;
    switch (equalityDefinition) {
        case 'default':
            equal = (a, b) => a === b;
            break;
        case 'shallow':
            equal = shallowEqual;
            break;
        case 'deep':
            equal = deepEqual;
            break;
    }
    return computed(computation, { equal });
}

/*
 * Public API Surface of ngx-immutable-signal
 */

/**
 * Generated bundle index. Do not edit.
 */

export { derived, immutableSignal };
//# sourceMappingURL=ngx-immutable-signal.mjs.map
